/*****************************************************************************
 * vout_dummy.c: Dummy video output display method for testing purposes
 *****************************************************************************
 * Copyright (C) 2000, 2001 VideoLAN
 * $Id: vout_dummy.c,v 1.10 2001/12/09 17:01:36 sam Exp $
 *
 * Authors: Samuel Hocevar <sam@zoy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#define MODULE_NAME dummy
#include "modules_inner.h"

/*****************************************************************************
 * Preamble
 *****************************************************************************/
#include "defs.h"

#include <errno.h>                                                 /* ENOMEM */
#include <stdlib.h>                                                /* free() */
#include <string.h>                                            /* strerror() */

#include "common.h"
#include "intf_msg.h"
#include "threads.h"
#include "mtime.h"
#include "tests.h"

#include "video.h"
#include "video_output.h"

#include "modules.h"
#include "modules_export.h"

#define DUMMY_WIDTH 16
#define DUMMY_HEIGHT 16
#define DUMMY_MAX_DIRECTBUFFERS 5

/*****************************************************************************
 * vout_sys_t: dummy video output method descriptor
 *****************************************************************************
 * This structure is part of the video output thread descriptor.
 * It describes the dummy specific properties of an output thread.
 *****************************************************************************/
typedef struct vout_sys_s
{
    /* Nothing needed here. Maybe stats ? */

} vout_sys_t;

/*****************************************************************************
 * Local prototypes
 *****************************************************************************/
static int  vout_Probe     ( probedata_t *p_data );
static int  vout_Create    ( struct vout_thread_s * );
static int  vout_Init      ( struct vout_thread_s * );
static void vout_End       ( struct vout_thread_s * );
static void vout_Destroy   ( struct vout_thread_s * );
static int  vout_Manage    ( struct vout_thread_s * );
static void vout_Display   ( struct vout_thread_s *, struct picture_s * );

/*****************************************************************************
 * Functions exported as capabilities. They are declared as static so that
 * we don't pollute the namespace too much.
 *****************************************************************************/
void _M( vout_getfunctions )( function_list_t * p_function_list )
{
    p_function_list->pf_probe = vout_Probe;
    p_function_list->functions.vout.pf_create     = vout_Create;
    p_function_list->functions.vout.pf_init       = vout_Init;
    p_function_list->functions.vout.pf_end        = vout_End;
    p_function_list->functions.vout.pf_destroy    = vout_Destroy;
    p_function_list->functions.vout.pf_manage     = vout_Manage;
    p_function_list->functions.vout.pf_display    = vout_Display;
    p_function_list->functions.vout.pf_setpalette = NULL;
}

/*****************************************************************************
 * intf_Probe: return a score
 *****************************************************************************/
static int vout_Probe( probedata_t *p_data )
{
    if( TestMethod( VOUT_METHOD_VAR, "dummy" ) )
    {
        return( 999 );
    }

    return( 1 );
}

/*****************************************************************************
 * vout_Create: allocates dummy video thread output method
 *****************************************************************************
 * This function allocates and initializes a dummy vout method.
 *****************************************************************************/
static int vout_Create( vout_thread_t *p_vout )
{
    /* Allocate structure */
    p_vout->p_sys = malloc( sizeof( vout_sys_t ) );
    if( p_vout->p_sys == NULL )
    {
        intf_ErrMsg("error: %s", strerror(ENOMEM) );
        return( 1 );
    }

    return( 0 );
}

/*****************************************************************************
 * vout_Init: initialize dummy video thread output method
 *****************************************************************************/
static int vout_Init( vout_thread_t *p_vout )
{
    picture_t *p_pic;
    int        i_index = 0;
    int        i_luma_bytes, i_chroma_bytes;

    /* Try to initialize DUMMY_MAX_DIRECTBUFFERS direct buffers */
    while( i_index < DUMMY_MAX_DIRECTBUFFERS )
    {
        p_pic = &p_vout->p_picture[ i_index ];

        switch( p_vout->i_chroma )
        {
        /* We know this chroma, allocate a buffer which will be used
         * directly by the decoder */
        case YUV_420_PICTURE:

            p_pic->i_chroma = YUV_420_PICTURE;
            p_pic->i_width  = p_vout->i_width;
            p_pic->i_height = p_vout->i_height;

            /* Precalculate some values */
            p_pic->i_size         = p_vout->i_width * p_vout->i_height;
            p_pic->i_chroma_width = p_vout->i_width / 2;
            p_pic->i_chroma_size  = p_vout->i_width * p_vout->i_height / 2;

            /* Allocate the memory buffer */
            i_luma_bytes = p_pic->i_size * sizeof(pixel_data_t);
            i_chroma_bytes = p_pic->i_chroma_size * sizeof(pixel_data_t);

            /* Y buffer */
            p_pic->planes[ Y_PLANE ].p_data = malloc( i_luma_bytes + 2 * i_chroma_bytes );
            p_pic->planes[ Y_PLANE ].i_bytes = i_luma_bytes;

            /* U buffer */
            p_pic->planes[ U_PLANE ].p_data = p_pic->planes[ Y_PLANE ].p_data + p_pic->i_height * p_pic->i_width;
            p_pic->planes[ U_PLANE ].i_bytes = i_chroma_bytes;

            /* V buffer */
            p_pic->planes[ V_PLANE ].p_data = p_pic->planes[ U_PLANE ].p_data + p_pic->i_height * p_pic->i_chroma_width;
            p_pic->planes[ V_PLANE ].i_bytes = i_chroma_bytes;

            /* We allocated 3 planes */
            p_pic->i_planes = 3;

            break;

        /* Unknown chroma, allocate an RGB buffer, the video output's job
         * will be to do the chroma->RGB conversion */
        default:

            p_pic->i_chroma = RGB_16BPP_PICTURE;
            p_pic->i_width  = DUMMY_WIDTH;
            p_pic->i_height = DUMMY_HEIGHT;

            /* Precalculate some values */
            i_luma_bytes    = sizeof(u16) * DUMMY_WIDTH * DUMMY_HEIGHT;

            /* Allocate the memory buffer */
            p_pic->planes[ RGB_PLANE ].p_data = malloc( i_luma_bytes );
            p_pic->planes[ RGB_PLANE ].i_bytes = i_luma_bytes;

            /* We allocated 1 plane */
            p_pic->i_planes = 1;

            break;
        }

        if( p_pic->i_planes == 0 )
        {
            break;
        }

        p_pic->i_status        = DESTROYED_PICTURE;

        p_pic->b_directbuffer  = 1;

        p_pic->i_left_margin   =
        p_pic->i_right_margin  =
        p_pic->i_top_margin    =
        p_pic->i_bottom_margin = 0;

        i_index++;
    }

    /* How many directbuffers did we create ? */
    p_vout->i_directbuffers = i_index;

    return( 0 );
}

/*****************************************************************************
 * vout_End: terminate dummy video thread output method
 *****************************************************************************/
static void vout_End( vout_thread_t *p_vout )
{
    int i_index;

    /* Free the fake direct buffers we allocated */
    for( i_index = p_vout->i_directbuffers ; i_index ; )
    {
        i_index--;
        free( p_vout->p_picture[ i_index ].planes[ 0 ].p_data );
    }
}

/*****************************************************************************
 * vout_Destroy: destroy dummy video thread output method
 *****************************************************************************
 * Terminate an output method created by DummyCreateOutputMethod
 *****************************************************************************/
static void vout_Destroy( vout_thread_t *p_vout )
{
    free( p_vout->p_sys );
}

/*****************************************************************************
 * vout_Manage: handle dummy events
 *****************************************************************************
 * This function should be called regularly by video output thread. It manages
 * console events. It returns a non null value on error.
 *****************************************************************************/
static int vout_Manage( vout_thread_t *p_vout )
{
    return( 0 );
}

/*****************************************************************************
 * vout_Display: displays previously rendered output
 *****************************************************************************
 * This function send the currently rendered image to dummy image, waits until
 * it is displayed and switch the two rendering buffers, preparing next frame.
 *****************************************************************************/
static void vout_Display( vout_thread_t *p_vout, picture_t *p_pic )
{
    /* No need to do anything, the fake direct buffers stay as they are */
}

